@@ -47,11 +47,12 @@ from binascii import unhexlify
 from sqlalchemy import create_engine
 from linotp.lib.audit.base import AuditBase
 from pylons import config
-import logging
+
 import logging.config
-import logging.handlers
 import traceback
 
+import linotp
+
 # Create the logging object from the linotp.ini config file
 ini_file = config.get("__file__")
 logging.config.fileConfig(ini_file, disable_existing_loggers=False)
@@ -63,11 +64,11 @@ def now():
     u_now = u"%s" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
     return u_now
 
+######################## MODEL ################################################
 table_prefix = config.get("linotpAudit.sql.table_prefix", "")
+audit_table_name = '%saudit' % table_prefix
 
-######################## MODEL ##########################################################
-
-audit_table = schema.Table('%saudit' % table_prefix, metadata,
+audit_table = schema.Table(audit_table_name, metadata,
     schema.Column('id', types.Integer, schema.Sequence('audit_seq_id',
                                                        optional=True),
                   primary_key=True),
@@ -88,6 +89,11 @@ audit_table = schema.Table('%saudit' % table_prefix, metadata,
     schema.Column('clearance_level', types.Integer, default=0)
 )
 
+
+AUDIT_ENCODE = ["action", "serial", "success", "user", "realm", "tokentype",
+                "administrator", "action_detail", "info", "linotp_server",
+                "client", "log_level"]
+
 class AuditTable(object):
 
     def __init__(self, serial=u"", action=u"", success=u"False",
@@ -136,44 +142,90 @@ class AuditTable(object):
         log.debug("[__init__] creating AuditTable object, action = %s"
                   % action)
 
-        self.serial = unicode(serial)
-        self.action = unicode(action)
-        self.success = unicode(success)
-        self.tokentype = unicode(tokentype)
-        self.user = unicode(user)
-        self.realm = unicode(realm)
-        self.administrator = unicode(administrator)
-        self.action_detail = unicode(action_detail)
-        self.info = unicode(info)
-        self.linotp_server = unicode(linotp_server)
-        self.client = unicode(client)
-        self.log_level = unicode(log_level)
+        self.serial = unicode(serial or '')
+        self.action = unicode(action or '')
+        self.success = unicode(success or '0')
+        self.tokentype = unicode(tokentype or '')
+        self.user = unicode(user or '')
+        self.realm = unicode(realm or '')
+        self.administrator = unicode(administrator or '')
+        self.action_detail = unicode(action_detail or '')
+        self.info = unicode(info or '')
+        self.linotp_server = unicode(linotp_server or '')
+        self.client = unicode(client or '')
+        self.log_level = unicode(log_level or '')
         self.clearance_level = clearance_level
+        self.timestamp = now()
+        self.siganture = ' '
 
+    def _get_field_len(self, col_name):
+        leng = -1
+        try:
+            ll = audit_table.columns[col_name]
+            ty = ll.type
+            leng = ty.length
+        except Exception as exx:
+            leng = -1
 
-    def getAsString(self):
-        '''
-        We need to distinguish, if this is an entry after the adding the
-        client entry or before. Otherwise the old signatures will break!
-        '''
-        s = "number=%s, date=%s, action=%s, %s, serial=%s, %s, user=%s, %s, admin=%s, %s, %s, server=%s, %s, %s" % (
-                    str(self.id), str(self.timestamp), self.action, str(self.success),
-                    self.serial, self.tokentype, self.user, self.realm,
-                    self.administrator, self.action_detail, self.info,
-                    self.linotp_server, self.log_level, str(self.clearance_level))
+        return leng
 
-        if self.client:
-            s += ", client=%s" % self.client
-        return s
+    def __setattr__(self, name, value):
+        """
+        to support unicode on all backends, we use the json encoder with
+        the assci encode default
 
+        :param name: db column name or class memeber
+        :param value: the corresponding value
 
+        :return: - nothing -
+        """
+        if type(value) in [str, unicode]:
+            field_len = self._get_field_len(name)
+            encoded_value = linotp.lib.crypt.uencode(value)
+            if field_len != -1 and len(encoded_value) > field_len:
+                log.warning("truncating audit data: [audit.%s] %s" % (name, value))
+                trunc_as_err = config.get("linotpAudit.error_on_truncation", False) or False
+                if trunc_as_err != False:
+                    raise Exception("truncating audit data: [audit.%s] %s" % (name, value))
+
+                ## during the encoding the value might expand -
+                ## so we take this additional length into account
+                add_len = len(encoded_value) - len(value)
+                value = value[:field_len - add_len]
+
+        if name in AUDIT_ENCODE:
+            ## encode data
+            if value:
+                value = linotp.lib.crypt.uencode(value)
+        super(AuditTable, self).__setattr__(name, value)
+
+    def __getattribute__(self, name):
+        """
+        to support unicode on all backends, we use the json decoder with
+        the assci decode default
+
+        :param name: db column name or class memeber
+
+        :return: the corresponding value
+        """
+        #Default behaviour
+        value = object.__getattribute__(self, name)
+        if name in AUDIT_ENCODE:
+            if value:
+                value = linotp.lib.crypt.udecode(value)
+            else:
+                value = ""
+
+        return value
 
 orm.mapper(AuditTable, audit_table)
 
 ########################################################################################
 
 class Audit(AuditBase):
-
+    """
+    Audit Implementation to the generic audit interface
+    """
     def __init__(self):
         self.name = "SQlAudit"
 
@@ -187,18 +239,22 @@ class Audit(AuditBase):
         if implicit_returning:
             # If implicit_returning is explicitly set to True, we
             # get lots of mysql errors
-            # AttributeError: 'MySQLCompiler_mysqldb' object has no attribute 'returning_clause'
+            # AttributeError: 'MySQLCompiler_mysqldb' object has no
+            # attribute 'returning_clause'
             # So we do not mention explicit_returning at all
-            self.engine = create_engine(connect_string, pool_recycle=pool_recycle)
+            self.engine = create_engine(connect_string,
+                                        pool_recycle=pool_recycle)
         else:
-            self.engine = create_engine(connect_string, pool_recycle=pool_recycle, implicit_returning=False)
+            self.engine = create_engine(connect_string,
+                                        pool_recycle=pool_recycle,
+                                        implicit_returning=False)
 
         metadata.bind = self.engine
         metadata.create_all()
 
         # Set up the session
-        self.sm = orm.sessionmaker(bind=self.engine, autoflush=True, autocommit=True,
-            expire_on_commit=True)
+        self.sm = orm.sessionmaker(bind=self.engine, autoflush=True,
+                                   autocommit=True, expire_on_commit=True)
         self.session = orm.scoped_session(self.sm)
 
         # initialize signing keys
@@ -213,13 +269,43 @@ class Audit(AuditBase):
             # create the column "client"
             column = schema.Column("client", types.Unicode(80))
             column.create(audit_table)
-        except Exception, e:
+        except Exception as exx:
             # Obviously we already migrated the database.
-            log.info("[__init__] Error during database migration: %r" % e)
+            log.info("[__init__] Error during database migration: %r" % exx)
 
 
+    def _attr_to_dict(self, audit_line):
+
+        line = {}
+        line['number'] = audit_line.id
+        line['id'] = audit_line.id
+        line['date'] = str(audit_line.timestamp)
+        line['timestamp'] = str(audit_line.timestamp)
+        line['missing_line'] = ""
+        line['serial'] = audit_line.serial
+        line['action'] = audit_line.action
+        line['action_detail'] = audit_line.action_detail
+        line['success'] = audit_line.success
+        line['token_type'] = audit_line.tokentype
+        line['tokentype'] = audit_line.tokentype
+        line['user'] = audit_line.user
+        line['realm'] = audit_line.realm
+        line['administrator'] = audit_line.administrator
+        line['action_detail'] = audit_line.action_detail
+        line['info'] = audit_line.info
+        line['linotp_server'] = audit_line.linotp_server
+        line["client"] = audit_line.client
+        line['log_level'] = audit_line.log_level
+        line['clearance_level'] = audit_line.clearance_level
+
+        return line
+
     def _sign(self, audit_line):
-        s_audit = audit_line.getAsString()
+        '''
+        Create a signature of the audit object
+        '''
+        line = self._attr_to_dict(audit_line)
+        s_audit = getAsString(line)
         log.debug("[_sign] signing %s" % s_audit)
 
         key = EVP.load_key_string(self.private)
@@ -230,54 +316,88 @@ class Audit(AuditBase):
         log.debug("[_sign] signature : %s" % hexlify(signature))
         return hexlify(signature)
 
+
     def _verify(self, auditline, signature):
         '''
         Verify the signature of the audit line
         '''
-        s_audit = auditline.getAsString()
+        res = False
+        if not signature:
+            log.debug("[_verify] missing signature %r" % auditline)
+            return res
+
+        s_audit = getAsString(auditline)
         log.debug("[_verify] verifying %s" % s_audit)
+
         self.VerifyEVP.verify_init()
         self.VerifyEVP.verify_update(s_audit)
         res = self.VerifyEVP.verify_final(unhexlify(signature))
+
         return res
 
-
     def log(self, param):
+        '''
+        This method is used to log the data. It splits information of
+        multiple tokens (e.g from import) in multiple audit log entries
+        '''
+        log.debug("[log] writing audit log message")
+
+        try:
+            serial = param.get('serial', '') or ''
+            if not serial:
+                ## if no serial, do as before
+                self.log_entry(param)
+            else:
+                ## look if we have multiple serials inside
+                serials = serial.split(',')
+                for serial in serials:
+                    p = {}
+                    p.update(param)
+                    p['serial'] = serial
+                    self.log_entry(p)
+
+            #self.session.commit()
+            log.debug("[log] writing log done!")
+
+        except Exception as  exx:
+            log.error("[log] error writing log message: %r" % exx)
+            log.error("[log] %s" % traceback.format_exc())
+            self.session.rollback()
+            raise exx
+
+        finally:
+            log.debug("[log] writing log done!")
+
+        return
+
+    def log_entry(self, param):
         '''
         This method is used to log the data.
         It should hash the data and do a hash chain and sign the data
         '''
-        log.debug("[log] writing audit log message")
-        try:
-            at = AuditTable(
-                        serial=param.get('serial'),
-                        action=param.get('action'),
-                        success=1 if param.get('success') else 0,
-                        tokentype=param.get('token_type'),
-                        user=param.get('user'),
-                        realm=param.get('realm'),
-                        administrator=param.get('administrator'),
-                        action_detail=param.get('action_detail'),
-                        info=param.get('info'),
-                        linotp_server=param.get('linotp_server'),
-                        client=param.get('client'),
-                        log_level=param.get('log_level'),
-                        clearance_level=param.get('clearance_level')
+
+        at = AuditTable(
+                    serial=param.get('serial'),
+                    action=param.get('action'),
+                    success=1 if param.get('success') else 0,
+                    tokentype=param.get('token_type'),
+                    user=param.get('user'),
+                    realm=param.get('realm'),
+                    administrator=param.get('administrator'),
+                    action_detail=param.get('action_detail'),
+                    info=param.get('info'),
+                    linotp_server=param.get('linotp_server'),
+                    client=param.get('client'),
+                    log_level=param.get('log_level'),
+                    clearance_level=param.get('clearance_level')
             )
 
-            self.session.add(at)
-            self.session.flush()
-            # At this point "at" contains the primary key id
-            at.signature = self._sign(at)
-            self.session.merge(at)
-            self.session.flush()
-
-            #self.session.commit()
-        except Exception as  e:
-            log.error("[log] error writing log message: %s" % str(e))
-            log.error("[log] %s" % traceback.format_exc())
-            self.session.rollback()
-
+        self.session.add(at)
+        self.session.flush()
+        # At this point "at" contains the primary key id
+        at.signature = self._sign(at)
+        self.session.merge(at)
+        self.session.flush()
 
 
     def initialize_log(self, param):
@@ -306,7 +426,8 @@ class Audit(AuditBase):
         if not AND:
             boolCheck = or_
 
-        log.debug("[_buildCondition] building condition for params %s with %s" % (param, AND))
+        log.debug("[_buildCondition] building condition for params %s with %s"
+                  % (param, AND))
 
         for k, v in param.items():
             if "" != v:
@@ -361,33 +482,27 @@ class Audit(AuditBase):
         :return: audit entry dict
         """
 
-        line = {}
-        line['number'] = audit_line.id
-        line['date'] = str(audit_line.timestamp)
-        line['serial'] = audit_line.serial
-        line['action'] = audit_line.action
-        line['action_detail'] = audit_line.action_detail
-        line['success'] = audit_line.success
-        line['token_type'] = audit_line.tokentype
-        line['user'] = audit_line.user
-        line['realm'] = audit_line.realm
-        line['administrator'] = audit_line.administrator
-        line['action_detail'] = audit_line.action_detail
-        line['info'] = audit_line.info
-        line['linotp_server'] = audit_line.linotp_server
-        line["client"] = audit_line.client
-        line['log_level'] = audit_line.log_level
-        line['clearance_level'] = audit_line.clearance_level
+        line = self._attr_to_dict(audit_line)
+
+        ## if we have an \uencoded data, we extract the unicode back
+        for key, value in line.items():
+            if value and type(value) in [str, unicode]:
+                value = linotp.lib.crypt.udecode(value)
+                line[key] = value
+            elif value is None:
+                line[key] = ''
 
         # Signature check
         # TODO: use instead the verify_init
         log.debug("[search] old sig = %s" % audit_line.signature)
-        res = self._verify(audit_line, audit_line.signature)
+
+        res = self._verify(line, audit_line.signature)
         if res == 1:
             line['sig_check'] = "OK"
         else:
             line['sig_check'] = "FAIL"
 
+
         return line
 
     def searchQuery(self, param, AND=True, display_error=True, rp_dict=None):
@@ -398,7 +513,8 @@ class Audit(AuditBase):
             Search parameters can be passed.
 
         return:
-            An iterator is returned.
+            a result object which has to be converted with iter() to an
+            iterator
         '''
 
         if rp_dict is None:
@@ -471,146 +587,29 @@ class Audit(AuditBase):
 
         if 'rp' in rp_dict or 'page' in rp_dict:
             # build the LIMIT and OFFSET
-            limit = int(rp_dict.get('rp'))
-
-            if rp_dict.get('rp'):
-                limit = int(rp_dict.get('rp'))
+            page = 1
             offset = 0
-            if rp_dict.get('page'):
+            limit = 15
+
+            if 'rp' in rp_dict:
+                limit = int(rp_dict.get('rp'))
+
+            if 'page' in rp_dict:
                 page = int(rp_dict.get('page'))
-                offset = limit * (page - 1)
 
-                start = offset
-                stop = offset + limit
-                audit_q = audit_q.slice(start, stop)
-
-        return iter(audit_q)
-
-    def search(self, param, AND=True, display_error=True, rp_dict={}):
-        '''
-        This function is used to search audit events.
-
-        param:
-            Search parameters can be passed.
-
-        return:
-            A list of dictionaries is return.
-            Each list element denotes an audit event.
-        '''
-        if 'or' in param:
-            if "true" == param['or'].lower():
-                AND = False
-
-        result = [{}]
-
-        log.debug("[search] got the params %s" % param)
-        log.debug("[search] got the rp_dict %s" % rp_dict)
-
-        # build the condition / WHERE clause
-        condition = self._buildCondition(param, AND)
-        log.debug("[search] the following condition was build from "
-                  "the parameters %s (%s): %s" % (param, AND, condition))
-
-        # build the LIMIT and OFFSET
-        limit = 100
-        if rp_dict.get('rp'):
-            limit = int(rp_dict.get('rp'))
-        offset = 0
-        if rp_dict.get('page'):
-            page = int(rp_dict.get('page'))
             offset = limit * (page - 1)
 
-        order = AuditTable.id
-        if rp_dict.get("sortname"):
-            sortn = rp_dict.get('sortname').lower()
-            if "serial" == sortn:
-                order = AuditTable.serial
-            elif "number" == sortn:
-                order = AuditTable.id
-            elif "user" == sortn:
-                order = AuditTable.user
-            elif "action" == sortn:
-                order = AuditTable.action
-            elif "action_detail" == sortn:
-                order = AuditTable.action_detail
-            elif "realm" == sortn:
-                order = AuditTable.realm
-            elif "date" == sortn:
-                order = AuditTable.timestamp
-            elif "administrator" == sortn:
-                order = AuditTable.administrator
-            elif "success" == sortn:
-                order = AuditTable.success
-            elif "tokentype" == sortn:
-                order = AuditTable.tokentype
-            elif "info" == sortn:
-                order = AuditTable.info
-            elif "linotp_server" == sortn:
-                order = AuditTable.linotp_server
-            elif "client" == sortn:
-                order = AuditTable.client
-            elif "log_level" == sortn:
-                order = AuditTable.log_level
-            elif "clearance_level" == sortn:
-                order = AuditTable.clearance_level
+            start = offset
+            stop = offset + limit
+            audit_q = audit_q.slice(start, stop)
 
-        # build the ordering
-        order_dir = asc(order)
-
-        if rp_dict.get("sortorder"):
-            sorto = rp_dict.get('sortorder').lower()
-            if "desc" == sorto:
-                order_dir = desc(order)
-
-        if type(condition).__name__ == 'NoneType':
-            audit_q = self.session.query(AuditTable)\
-                .order_by(order_dir)
-        else:
-            audit_q = self.session.query(AuditTable)\
-                .filter(condition)\
-                .order_by(order_dir)
-
-        # FIXME? BUT THIS IS SO MUCH SLOWER!
-        # FIXME: Here desc() ordering also does not work! :/
-        start = offset
-        stop = offset + limit
-        audit_q = audit_q.slice(start, stop)
-
-        for audit_line in audit_q:
-            line = {}
-            line['number'] = audit_line.id
-            line['date'] = str(audit_line.timestamp)
-            line['serial'] = audit_line.serial
-            line['action'] = audit_line.action
-            line['action_detail'] = audit_line.action_detail
-            line['success'] = audit_line.success
-            line['token_type'] = audit_line.tokentype
-            line['user'] = audit_line.user
-            line['realm'] = audit_line.realm
-            line['administrator'] = audit_line.administrator
-            line['action_detail'] = audit_line.action_detail
-            line['info'] = audit_line.info
-            line['linotp_server'] = audit_line.linotp_server
-            line['client'] = audit_line.client
-            line['log_level'] = audit_line.log_level
-            line['clearance_level'] = audit_line.clearance_level
-
-            # Signature check
-            # TODO: use instead the verify_init
-            log.debug("[search] old sig = %s" % audit_line.signature)
-            res = self._verify(audit_line, audit_line.signature)
-            if res == 1:
-                line['sig_check'] = "OK"
-            else:
-                line['sig_check'] = "FAIL"
-            # TODO: missing line check
-
-
-            result.append(line)
-
-        log.debug("[search] %s" % result)
+        ## we drop here the ORM due to memory consumption
+        ## and return a resultproxy for row iteration
+        result = self.session.execute(audit_q.statement)
         return result
 
+
+
     def getTotal(self, param, AND=True, display_error=True):
         '''
         This method returns the total number of audit entries in
@@ -626,4 +625,24 @@ class Audit(AuditBase):
         log.debug("[getTotal] count=%s " % str(c))
         return c
 
+def getAsString(data):
+    '''
+    We need to distinguish, if this is an entry after the adding the
+    client entry or before. Otherwise the old signatures will break!
+    '''
+
+    s = "number=%s, date=%s, action=%s, %s, serial=%s, %s, user=%s, %s, admin=%s, %s, %s, server=%s, %s, %s" % (
+                str(data.get('id')), str(data.get('timestamp')),
+                data.get('action'), str(data.get('success')),
+                data.get('serial'), data.get('tokentype'),
+                data.get('user'), data.get('realm'),
+                data.get('administrator'), data.get('action_detail'),
+                data.get('info'), data.get('linotp_server'),
+                data.get('log_level'), str(data.get('clearance_level')))
+
+    if 'client' in data:
+        s += ", client=%s" % data.get('client')
+    return s
+
+
 ###eof#########################################################################
