@@ -30,15 +30,13 @@
 
 import binascii
 import logging
-import sys
 import traceback
-
 from datetime import datetime
 
-if sys.version_info[0:2] >= (2, 6):
-    from json import loads, dumps
-else:
-    from simplejson import loads, dumps
+try:
+    import json
+except ImportError:
+    import simplejson as json
 
 
 """The application's model objects"""
@@ -48,57 +46,49 @@ from sqlalchemy import orm
 from sqlalchemy import ForeignKey
 from sqlalchemy.orm import relation
 
+import linotp
+
 from linotp.model import meta
 from linotp.model.meta import Session
+from linotp.model.meta import MetaData
+
 from linotp.lib.crypt import geturandom
 from linotp.lib.crypt import encrypt, hash, SecretObj
 from linotp.lib.crypt import encryptPin
 from linotp.lib.crypt import decryptPin
 from linotp.lib.crypt import get_rand_digit_str
 
-log = logging.getLogger(__name__)
 
 from pylons import config
+log = logging.getLogger(__name__)
+
 implicit_returning = config.get('linotpSQL.implicit_returning', True)
 
+## for oracle we need a mapping of columns
+## due to reserved keywords 'session' and 'timestamp'
+COL_PREFIX = ""
+SQLU = config.get("sqlalchemy.url", "")
+if SQLU.startswith("oracle:"):
+    COL_PREFIX = config.get("oracle.sql.column_prefix", "lino")
+
+
+session_column = "%ssession" % COL_PREFIX
+timestamp_column = "%stimestamp" % COL_PREFIX
 
 def init_model(engine):
-    """Call me before using any of the tables or classes in the model"""
-    ## Reflected tables must be defined and mapped here
-    #global reflected_table
-    #reflected_table = sa.Table("Reflected", meta.metadata, autoload=True,
-    #                           autoload_with=engine)
-    #orm.mapper(Reflected, reflected_table)
-    #
-    #sm = orm.sessionmaker(autoflush=True, autocommit=False, bind=engine)
-
-    #meta.Session = orm.scoped_session(sm)
-    #meta.engine = engine
-
-    meta.Session.configure(bind=engine)
-    meta.engine = engine
-
+    """
+    init_model binds the table objects to the class objects
+    - to be called before using any of the tables or classes in the model!!!
 
+    :param engine: the sql engine
+    """
     log.debug('model init: init_model')
 
-## Non-reflected tables may be defined and mapped at module level
-#foo_table = sa.Table("Foo", meta.metadata,
-#    sa.Column("id", sa.types.Integer, primary_key=True),
-#    sa.Column("bar", sa.types.String(255), nullable=False),
-#    )
-#
-#class Foo(object):
-#    pass
-#
-#orm.mapper(Foo, foo_table)
+    meta.engine = engine
+    meta.Session.configure(bind=engine)
 
-
-## Classes for reflected tables may be defined here, but the table and
-## mapping itself must be done in the init_model function
-#reflected_table = None
-#
-#class Reflected(object):
-#    pass
+    log.debug('model init: init_model')
+    return
 
 token_table = sa.Table('Token', meta.metadata,
                 sa.Column('LinOtpTokenId', sa.types.Integer(), sa.Sequence('token_seq_id', optional=True), primary_key=True, nullable=False),
@@ -132,7 +122,9 @@ token_table = sa.Table('Token', meta.metadata,
                 implicit_returning=implicit_returning,
                 )
 
-
+TOKEN_ENCODE = ["LinOtpTokenDesc", "LinOtpTokenSerialnumber",
+                "LinOtpTokenInfo", "LinOtpUserid", "LinOtpIdResClass",
+                "LinOtpIdResolver"]
 
 class Token(object):
 
@@ -162,6 +154,42 @@ class Token(object):
         # self.LinOtpTokenId      = 0
         log.debug('Token init done')
 
+    def __setattr__(self, name, value):
+        """
+        to support unicode on all backends, we use the json encoder with
+        the assci encode default
+
+        :param name: db column name or class memeber
+        :param value: the corresponding value
+
+        :return: - nothing -
+        """
+        if name in TOKEN_ENCODE:
+            ## encode data
+            if value:
+                value = linotp.lib.crypt.uencode(value)
+        super(Token, self).__setattr__(name, value)
+
+    def __getattribute__(self, name):
+        """
+        to support unicode on all backends, we use the json decoder with
+        the assci decode default
+
+        :param name: db column name or class memeber
+
+        :return: the corresponding value
+        """
+        #Default behaviour
+        value = object.__getattribute__(self, name)
+        if name in TOKEN_ENCODE:
+            if value:
+                value = linotp.lib.crypt.udecode(value)
+            else:
+                value = ""
+
+        return value
+
+
     def _fix_spaces(self, data):
         '''
         On MS SQL server empty fields ("") like the LinOtpTokenInfo
@@ -202,21 +230,18 @@ class Token(object):
 
     def getHOtpKey(self):
         log.debug('getHOtpKey()')
-        key = binascii.unhexlify(self.LinOtpKeyEnc)
-        iv = binascii.unhexlify(self.LinOtpKeyIV)
+        key = binascii.unhexlify(self.LinOtpKeyEnc or '')
+        iv = binascii.unhexlify(self.LinOtpKeyIV or '')
         secret = SecretObj(key, iv)
         return secret
 
     def getOtpCounter(self):
-        return self.LinOtpCount
+        return self.LinOtpCount or 0
 
     def getUserPin(self):
         log.debug('getHOtpKey()')
-        pu = self.LinOtpTokenPinUser
-        if pu is None: pu = ''
-        puiv = self.LinOtpTokenPinUserIV
-        if puiv is None: puiv = ''
-
+        pu = self.LinOtpTokenPinUser or ''
+        puiv = self.LinOtpTokenPinUserIV or ''
         key = binascii.unhexlify(pu)
         iv = binascii.unhexlify(puiv)
         secret = SecretObj(key, iv)
@@ -236,10 +261,11 @@ class Token(object):
         ## calculate a hash from a pin
         # Fix for working with MS SQL servers
         # MS SQL servers sometimes return a '<space>' when the column is empty: ''
-        seed_str = self._fix_spaces(self.LinOtpSeed)
+        seed_str = self._fix_spaces(self.LinOtpSeed or '')
         seed = binascii.unhexlify(seed_str)
         hPin = hash(pin, seed)
-        log.debug("[getHashedPin] hPin: %s, pin: %s, seed: %s" % (binascii.hexlify(hPin), pin, self.LinOtpSeed))
+        log.debug("[getHashedPin] hPin: %s, pin: %s, seed: %s" %
+                  (binascii.hexlify(hPin), pin, self.LinOtpSeed or ''))
         return binascii.hexlify(hPin)
 
     def setDescription(self, desc):
@@ -277,23 +303,23 @@ class Token(object):
             log.error("[comparePin] no valid PIN!")
             return res
 
-        if (self.isPinEncrypted() == True):
-            log.debug("[comparePin] we got an encrypted PIN!")
-            tokenPin = self.LinOtpPinHash[2:]
-            decryptTokenPin = decryptPin(tokenPin)
-            # TODO CKO: remove the TokenPin
-            #log.debug("[comparePin] the decrypted PIN is %s" % decryptTokenPin)
-            if (decryptTokenPin == pin):
-                res = True
-        else:
-            log.debug("[comparePin] we got a hashed PIN!")
-            # TODO CKO: remove the PIN hash
-            #log.debug("[comparePin] The Hash is %s while the LinOtpPinHash is %s" % (mypHash,self.LinOtpPinHash))
-            if len(self.LinOtpPinHash) > 0:
-                mypHash = self.getHashedPin(pin)
+        if self.LinOtpPinHash:
+            if self.isPinEncrypted():
+                log.debug("[comparePin] we got an encrypted PIN!")
+                tokenPin = self.LinOtpPinHash[2:]
+                decryptTokenPin = decryptPin(tokenPin)
+                if decryptTokenPin == pin:
+                    res = True
             else:
-                mypHash = pin
-            if (mypHash == self.LinOtpPinHash):
+                log.debug("[comparePin] we got a hashed PIN!")
+                ## is there a hashed pin
+                hashed_pin = self.getHashedPin(pin)
+                if hashed_pin == self.LinOtpPinHash:
+                    res = True
+
+        else: ## token hashed pin is empyt or none
+            log.debug("[comparePin] there is no pin for this token!")
+            if pin == '':
                 res = True
 
         return res
@@ -313,7 +339,7 @@ class Token(object):
         ret = False
         if pin is None:
             pin = self.LinOtpPinHash
-        if (pin.startswith("@@") == True):
+        if pin and pin.startswith("@@"):
             ret = True
         return ret
 
@@ -356,7 +382,7 @@ class Token(object):
             if hasattr(self, kMethod):
                 return getattr(self, kMethod)()
             else:
-                return getattr(self, key)
+                return getattr(self, key) or ''
         else:
             return fallback
 
@@ -365,19 +391,19 @@ class Token(object):
         log.debug('get_vars()')
 
         ret = {}
-        ret['LinOtp.TokenId'] = self.LinOtpTokenId
-        ret['LinOtp.TokenDesc'] = self.LinOtpTokenDesc
-        ret['LinOtp.TokenSerialnumber'] = self.LinOtpTokenSerialnumber
+        ret['LinOtp.TokenId'] = self.LinOtpTokenId or ''
+        ret['LinOtp.TokenDesc'] = self.LinOtpTokenDesc or ''
+        ret['LinOtp.TokenSerialnumber'] = self.LinOtpTokenSerialnumber or ''
 
-        ret['LinOtp.TokenType'] = self.LinOtpTokenType
-        ret['LinOtp.TokenInfo'] = self._fix_spaces(self.LinOtpTokenInfo)
+        ret['LinOtp.TokenType'] = self.LinOtpTokenType or 'hmac'
+        ret['LinOtp.TokenInfo'] = self._fix_spaces(self.LinOtpTokenInfo or '')
         # ret['LinOtpTokenPinUser']   = self.LinOtpTokenPinUser
         # ret['LinOtpTokenPinSO']     = self.LinOtpTokenPinSO
 
-        ret['LinOtp.IdResolver'] = self.LinOtpIdResolver
-        ret['LinOtp.IdResClass'] = self.LinOtpIdResClass
-        ret['LinOtp.Userid'] = self.LinOtpUserid
-        ret['LinOtp.OtpLen'] = self.LinOtpOtpLen
+        ret['LinOtp.IdResolver'] = self.LinOtpIdResolver or ''
+        ret['LinOtp.IdResClass'] = self.LinOtpIdResClass or ''
+        ret['LinOtp.Userid'] = self.LinOtpUserid or ''
+        ret['LinOtp.OtpLen'] = self.LinOtpOtpLen or 6
         # ret['LinOtp.PinHash']        = self.LinOtpPinHash
 
         ret['LinOtp.MaxFail'] = self.LinOtpMaxFail
@@ -421,7 +447,7 @@ class Token(object):
     def getInfo(self):
         # Fix for working with MS SQL servers
         # MS SQL servers sometimes return a '<space>' when the column is empty: ''
-        return self._fix_spaces(self.LinOtpTokenInfo)
+        return self._fix_spaces(self.LinOtpTokenInfo or '')
 
     def setInfo(self, info):
         self.LinOtpTokenInfo = info
@@ -434,6 +460,13 @@ class Token(object):
             self.setPin(pin, hashed)
 
     def storeToken(self):
+        if self.LinOtpUserid is None:
+            self.LinOtpUserid = u''
+        if self.LinOtpIdResClass is None:
+            self.LinOtpIdResClass = ''
+        if self.LinOtpIdResolver is None:
+            self.LinOtpIdResolver = ''
+
         log.debug('storeToken()')
         Session.add(self)
         Session.flush()
@@ -446,7 +479,7 @@ class Token(object):
         return
 
     def getType(self):
-        return self.LinOtpTokenType
+        return self.LinOtpTokenType or 'hmac'
 
     def updateType(self, typ):
         #in case the prevoius has been different type
@@ -475,7 +508,7 @@ class Token(object):
         self.updateOtpKey(otpKey)
 
     def getRealms(self):
-        return self.realms
+        return self.realms or ''
 
     def getRealmNames(self):
         r_list = []
@@ -503,7 +536,7 @@ def createToken(serial):
 
     return token
 
-
+###############################################################################
 
 config_table = sa.Table('Config', meta.metadata,
                 sa.Column('Key', sa.types.Unicode(255), primary_key=True, nullable=False),
@@ -515,6 +548,8 @@ config_table = sa.Table('Config', meta.metadata,
 
 log.debug('config table append_column')
 
+CONFIG_ENCODE = ["Key", "Value", "Description"]
+
 class Config(object):
 
     def __init__(self, Key, Value, Type=u'', Description=u''):
@@ -534,6 +569,41 @@ class Config(object):
     def __unicode__(self):
         return self.Description
 
+    def __setattr__(self, name, value):
+        """
+        to support unicode on all backends, we use the json encoder with
+        the assci encode default
+
+        :param name: db column name or class memeber
+        :param value: the corresponding value
+
+        :return: - nothing -
+        """
+        if name in CONFIG_ENCODE:
+            ## encode data
+            if value:
+                value = linotp.lib.crypt.uencode(value)
+        super(Config, self).__setattr__(name, value)
+
+    def __getattribute__(self, name):
+        """
+        to support unicode on all backends, we use the json decoder with
+        the assci decode default
+
+        :param name: db column name or class memeber
+
+        :return: the corresponding value
+        """
+        #Default behaviour
+        value = object.__getattribute__(self, name)
+        if name in CONFIG_ENCODE:
+            if value:
+                value = linotp.lib.crypt.udecode(value)
+            else:
+                value = ""
+
+        return value
+
     __str__ = __unicode__
 
 
@@ -562,13 +632,56 @@ realm_table = sa.Table('Realm', meta.metadata,
                 implicit_returning=implicit_returning,
                 )
 
+REALM_ENCODE = ["name", "option"]
+
 class Realm(object):
+
+    def __setattr__(self, name, value):
+        """
+        to support unicode on all backends, we use the json encoder with
+        the assci encode default
+
+        :param name: db column name or class memeber
+        :param value: the corresponding value
+
+        :return: - nothing -
+        """
+        if name in REALM_ENCODE:
+            ## encode data
+            if value:
+                value = linotp.lib.crypt.uencode(value)
+        super(Realm, self).__setattr__(name, value)
+
+    def __getattribute__(self, name):
+        """
+        to support unicode on all backends, we use the json decoder with
+        the assci decode default
+
+        :param name: db column name or class memeber
+
+        :return: the corresponding value
+        """
+        #Default behaviour
+        value = object.__getattribute__(self, name)
+        if name in REALM_ENCODE:
+            if value:
+                value = linotp.lib.crypt.udecode(value)
+            else:
+                value = ""
+
+        return value
+
     def __init__(self, realm):
         log.debug("setting realm name to %s" % realm)
         self.name = realm
+        if realm is not None:
+            self.name = realm.lower()
         #self.id     = 0
 
     def storeRealm(self):
+        if self.name is None:
+            self.name = ''
+        self.name = self.name.lower()
         log.debug('storeRealm()')
         Session.add(self)
         Session.commit()
@@ -582,21 +695,24 @@ ocra challenges are stored
 
 log.debug('creating ocra table')
 
+
 ocra_table = sa.Table('ocra', meta.metadata,
                 sa.Column('id', sa.types.Integer(), sa.Sequence('token_seq_id', optional=True), primary_key=True, nullable=False),
                 sa.Column('transid', sa.types.Unicode(20), unique=True,
                                                 nullable=False, index=True),
                 sa.Column('data', sa.types.Unicode(512), default=u''),
                 sa.Column('challenge', sa.types.Unicode(256), default=u''),
-                sa.Column('session', sa.types.Unicode(512), default=u''),
+                sa.Column(session_column, sa.types.Unicode(512), default=u''),
                 sa.Column('tokenserial', sa.types.Unicode(64), default=u''),
-                sa.Column('timestamp', sa.types.DateTime, default=datetime.now()),
+                sa.Column(timestamp_column, sa.types.DateTime, default=datetime.now()),
                 sa.Column('received_count', sa.types.Integer(), default=0),
                 sa.Column('received_tan', sa.types.Boolean, default=False),
                 sa.Column('valid_tan', sa.types.Boolean, default=False),
                 implicit_returning=implicit_returning,
                 )
 
+OCRA_ENCODE = ["data", "challenge", "tokenserial"]
+
 class OcraChallenge(object):
     '''
     '''
@@ -615,6 +731,41 @@ class OcraChallenge(object):
 
         log.debug('OcraChallenge: init done!')
 
+    def __setattr__(self, name, value):
+        """
+        to support unicode on all backends, we use the json encoder with
+        the assci encode default
+
+        :param name: db column name or class memeber
+        :param value: the corresponding value
+
+        :return: - nothing -
+        """
+        if name in OCRA_ENCODE:
+            ## encode data
+            if value:
+                value = linotp.lib.crypt.uencode(value)
+        super(OcraChallenge, self).__setattr__(name, value)
+
+    def __getattribute__(self, name):
+        """
+        to support unicode on all backends, we use the json decoder with
+        the assci decode default
+
+        :param name: db column name or class memeber
+
+        :return: the corresponding value
+        """
+        #Default behaviour
+        value = object.__getattribute__(self, name)
+        if name in OCRA_ENCODE:
+            if value:
+                value = linotp.lib.crypt.udecode(value)
+            else:
+                value = ""
+
+        return value
+
     def setData(self, data):
         self.data = unicode(data)
 
@@ -666,6 +817,7 @@ class OcraChallenge(object):
 
         return "%s" % unicode(descr)
 
+
     __str__ = __unicode__
 
 
@@ -685,9 +837,9 @@ challenges_table = sa.Table('challenges', meta.metadata,
                                                 index=True),
                 sa.Column('data', sa.types.Unicode(512), default=u''),
                 sa.Column('challenge', sa.types.Unicode(512), default=u''),
-                sa.Column('session', sa.types.Unicode(512), default=u''),
+                sa.Column(session_column, sa.types.Unicode(512), default=u''),
                 sa.Column('tokenserial', sa.types.Unicode(64), default=u''),
-                sa.Column('timestamp', sa.types.DateTime,
+                sa.Column(timestamp_column, sa.types.DateTime,
                                                     default=datetime.now()),
                 sa.Column('received_count', sa.types.Integer(), default=0),
                 sa.Column('received_tan', sa.types.Boolean, default=False),
@@ -695,6 +847,8 @@ challenges_table = sa.Table('challenges', meta.metadata,
                 implicit_returning=implicit_returning,
                 )
 
+CHALLENGE_ENCODE = ["data", "challenge", 'tokenserial']
+
 class Challenge(object):
     '''
     the generic challange handling
@@ -714,16 +868,62 @@ class Challenge(object):
 
         log.debug('Challenge: init done!')
 
+    def __setattr__(self, name, value):
+        """
+        to support unicode on all backends, we use the json encoder with
+        the assci encode default
+
+        :param name: db column name or class memeber
+        :param value: the corresponding value
+
+        :return: - nothing -
+        """
+        if name in CHALLENGE_ENCODE:
+            ## encode data
+            if value:
+                value = linotp.lib.crypt.uencode(value)
+        super(Challenge, self).__setattr__(name, value)
+
+    def __getattribute__(self, name):
+        """
+        to support unicode on all backends, we use the json decoder with
+        the assci decode default
+
+        :param name: db column name or class memeber
+
+        :return: the corresponding value
+        """
+        #Default behaviour
+        value = object.__getattribute__(self, name)
+        if name in CHALLENGE_ENCODE:
+            if value:
+                value = linotp.lib.crypt.udecode(value)
+            else:
+                value = ""
+
+        return value
+
+
+
     @classmethod
     def createTransactionId(cls , length=20):
         return get_rand_digit_str(length)
 
     def setData(self, data):
         if type(data) in [dict, list]:
-            self.data = dumps(data)
+            self.data = json.dumps(data)
         else:
             self.data = unicode(data)
 
+    def getData(self):
+        data = {}
+        try:
+            data = json.loads(self.data)
+        except:
+            data = self.data
+        return data
+
+
     def get(self, key=None, fallback=None, save=False):
         '''
         simulate the dict behaviour to make challenge processing
@@ -751,13 +951,6 @@ class Challenge(object):
     def getId(self):
         return self.id
 
-    def getData(self):
-        data = {}
-        try:
-            data = loads(self.data)
-        except:
-            data = self.data
-        return data
 
     def getSession(self):
         return self.session
@@ -858,5 +1051,29 @@ orm.mapper(Token, token_table, properties={
 orm.mapper(Realm, realm_table)
 orm.mapper(TokenRealm, tokenrealm_table)
 orm.mapper(Config, config_table)
-orm.mapper(OcraChallenge, ocra_table)
-orm.mapper(Challenge, challenges_table)
+
+
+## for oracle and the SQLAlchemy 0.7 we need a mapping of columns
+## due to reserved keywords session and timestamp
+mapping = {}
+mapping['session'] = "%ssession" % COL_PREFIX
+mapping['timestamp'] = "%stimestamp" % COL_PREFIX
+
+## create challenges ORM mapping to the Challenge class
+
+challenge_properties = {}
+if len(COL_PREFIX) > 0:
+    for key, value in mapping.items():
+        challenge_properties[key] = challenges_table.columns[value]
+
+orm.mapper(Challenge, challenges_table, properties=challenge_properties)
+
+## create Ocra ORM mapping to the Ocra class
+ocra_properties = {}
+if len(COL_PREFIX) > 0:
+    for key, value in mapping.items():
+        ocra_properties[key] = ocra_table.columns[value]
+
+orm.mapper(OcraChallenge, ocra_table, properties=ocra_properties)
+
+##eof#########################################################################
