@@ -1,21 +1,32 @@
-'''
-
-  product:  LinOTP2
-  module:   linotpd
-  edition:  Community Edition
-
-  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
-  License:  AGPLv3
-  contact:  http://www.linotp.org
-            http://www.lsexperts.de
-            linotp@lsexperts.de
-
-  Description:  This file is part of the linotpd service
-
-  Dependencies: -
+# -*- coding: utf-8 -*-
+#
+#    LinOTP - the open source solution for two factor authentication
+#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
+#
+#    This file is part of LinOTP server.
+#
+#    This program is free software: you can redistribute it and/or
+#    modify it under the terms of the GNU Affero General Public
+#    License, version 3, as published by the Free Software Foundation.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU Affero General Public License for more details.
+#
+#    You should have received a copy of the
+#               GNU Affero General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+#
+#    E-mail: linotp@lsexperts.de
+#    Contact: www.linotp.org
+#    Support: www.lsexperts.de
+#
 
 
-'''
+"""
+"""
 
 import logging
 
